import { URLExt } from '@jupyterlab/coreutils';
import { ServerConnection } from '@jupyterlab/services';
export async function submit(cellContents) {
    const endPoint = 'submit';
    const init = {
        method: 'POST',
        body: JSON.stringify({
            codeContent: cellContents
        })
    };
    try {
        return request(endPoint, init);
    }
    catch (error) {
        console.error(error);
        return { result: false };
    }
}
export async function checkStatus() {
    return request('status');
}
export async function isAuthenticated() {
    const response = await request('auth/authenticated');
    return response.result;
}
export function getEndpointUrl(endPoint) {
    const settings = ServerConnection.makeSettings();
    return URLExt.join(settings.baseUrl, 'ballet', endPoint);
}
/**
 * Call the API extension
 *
 * @param endPoint API REST end point for the extension
 * @param init Initial values for the request
 * @returns The response body interpreted as JSON
 */
export async function request(endPoint = '', init = {}) {
    // Make request to Jupyter API
    const requestUrl = getEndpointUrl(endPoint);
    let response;
    try {
        const settings = ServerConnection.makeSettings();
        response = await ServerConnection.makeRequest(requestUrl, init, settings);
    }
    catch (error) {
        throw new ServerConnection.NetworkError(error);
    }
    const data = await response.json();
    if (!response.ok) {
        throw new ServerConnection.ResponseError(response, data.message);
    }
    return data;
}
